/*
 * @psiAvailablePanel.java Version 1.1 06/01/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.Observable;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import seismic.seisStruct;        // Seismic Data Structure

/** Class psiAvailablePanel
 *  <p> This Class will build the Available Plot Tracks Panel that will allow
 *      the user to turn on or off any track that can be displayed.
 *
 *  @version 1.1 06/01/2016
 *  @author  John R. Victorine
 */

public class psiAvailablePanel extends JPanel implements ActionListener
{
  private Observable notifier = null;
  private int        iAction  = -1;

  public static final int OFF = 0;
  public static final int ON  = 1;

  public static final int _SOURCE         = seismic.seisTracksStruct._SOURCE;
  public static final int _TYPE           = seismic.seisTracksStruct._TYPE;
  public static final int _DESCRIPTION    = seismic.seisTracksStruct._DESCRIPTION;
  public static final int _STATUS         = seismic.seisTracksStruct._STATUS;

  public static final int _SRC_PSI        = seismic.seisTracksStruct._SRC_PSI;
  public static final int _SRC_SEISMIC    = seismic.seisTracksStruct._SRC_SEISMIC;

  public static final int _SRC_ID         = seismic.seisTracksStruct._SRC_ID;
  public static final int _SRC_COLUMN     = seismic.seisTracksStruct._SRC_COLUMN;

  // Plot Tracks

  private static final int _SEISMIC       = seismic.seisTracksStruct._SEISMIC;
  private static final int _SEISMIC_AVG   = seismic.seisTracksStruct._SEISMIC_AVG;
  private static final int _PRESSURE_TEST = seismic.seisTracksStruct._PRESSURE_TEST;
  private static final int _PRESSURE_AVG  = seismic.seisTracksStruct._PRESSURE_AVG;
  private static final int TOTAL_TRACKS   = seismic.seisTracksStruct.TOTAL_TRACKS;

  // Track Selection

  private int          iTrack      = -1;   // The Track that was altered

  private int          iSelect     = 0;    // Number of tracks selected
  private int          iSelected[] = null; // Tracks selected for view

  // Available Panel Widget Variables

  private int          iRows   = 0;  // Total Number of Rows in Panel
  private int          ilbl    = 0;  // Total Number of Labels
  private int          irb     = 0;  // Total Number of Radio Buttons

  private int          iStart = 0;
  private int          iEnd   = 0;
  private int          iTotal = 0;

  private JRadioButton rbS     = new JRadioButton();
  private JRadioButton rbD     = new JRadioButton();

  private JPanel       panel   = null;
  private JPanel       pnl[]   = null;
  private JLabel       lbl[]   = null; // Array of Labels to group the Tracks by
  private JRadioButton rb[]    = null;

  /** Constructor psiAvailablePanel()
   *  <p> This is the Constructor for this class.
   *  @param notifier = Observable
   */

  public psiAvailablePanel(Observable notifier)
  {
    try
    {
      this.notifier = notifier;
      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int i=0;

    JPanel      pnlTrk   = new JPanel();
    JPanel      pnlBase  = new JPanel();
    JPanel      pnlType  = new JPanel();
    JPanel      pnl1st   = new JPanel();
    JPanel      pnl3rd   = new JPanel();
    JPanel      pnl2nd   = new JPanel();
    JScrollPane scroll   = new JScrollPane();
    ButtonGroup group    = new ButtonGroup();
    ButtonGroup group1st = new ButtonGroup();
    ButtonGroup group2nd = new ButtonGroup();

    TitledBorder titledBorder = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white,new Color(165, 163, 151)),
        "Track Order");
    titledBorder.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    try
    {
      this.setLayout(new BorderLayout());
      buildPanel();
      panel.setBorder(titledBorder);

      this.add(scroll, BorderLayout.CENTER);
      scroll.getViewport().add(panel, null);
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method buildPanel()
   * <p> This method will build the Available Tracks Panel
   * @return  panel    = the Available Tracks Panel
   * @throws Exception
   */

  public JPanel buildPanel() throws Exception
  {
    String source = "";
    ButtonGroup group = new ButtonGroup();

    iRows   = seismic.seisTracksStruct.TOTAL_TRACKS +
              seismic.seisTracksStruct.TOTAL_SOURCES;

    panel = new JPanel();

    panel.setLayout(new GridLayout(iRows,1));
    panel.setSize(100, iRows*20);

    pnl     = new JPanel[seismic.seisTracksStruct.TOTAL_SOURCES];
    lbl     = new JLabel[seismic.seisTracksStruct.TOTAL_SOURCES];
    rb      = new JRadioButton[seismic.seisTracksStruct.TOTAL_TRACKS];

    for (int i=0; i<seismic.seisTracksStruct.TOTAL_TRACKS; i++)
    {
      if (!source.equals(seismic.seisTracksStruct.TRACK[i][_SOURCE]))
      {
        source = seismic.seisTracksStruct.TRACK[i][_SOURCE];

        pnl[ilbl] = new JPanel();
        pnl[ilbl].setBorder(BorderFactory.createEtchedBorder());

        lbl[ilbl] = new JLabel();
        lbl[ilbl].setFont(new java.awt.Font("Dialog", 1, 11));

        for (int j=0; j<seismic.seisTracksStruct.TOTAL_SOURCES; j++)
        {
          if (source.equals(seismic.seisTracksStruct.SOURCE[j][_SRC_ID]))
          {
            lbl[ilbl].setText(seismic.seisTracksStruct.SOURCE[j][_SRC_COLUMN]);
            pnl[ilbl].setBackground(
                new Color(seismic.seisTracksStruct.COLORS[j][0],
                          seismic.seisTracksStruct.COLORS[j][1],
                          seismic.seisTracksStruct.COLORS[j][2]));
          }
        }
        panel.add(pnl[ilbl], null);
        pnl[ilbl].add(lbl[ilbl], null);
        ilbl++;
      }

      rb[irb] = new JRadioButton();
      rb[irb].setSelected(false);
      rb[irb].setEnabled(false);

      rb[irb].setFont(new java.awt.Font("Dialog", 0, 11));
      rb[irb].setText(seismic.seisTracksStruct.TRACK[i][_DESCRIPTION]);
      rb[irb].addActionListener(this);

      panel.add(rb[irb], null);
      irb++;
    }

    return (panel);
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setSelected()
   * <p> This method will set the tracks that should be selected.
   * @param iSelected = Array of selected tracks
   */

  public void setSelected(int iSelected[])
  {
    int i = 0;

    this.iSelected = iSelected;

    for (i=0; i<seismic.seisTracksStruct.TOTAL_TRACKS; i++)
    {
      rb[i].setSelected(false);
    }

    if (iSelected != null)
    {
      for (i=0; i<iSelected.length; i++)
      {
        if (iSelected[i] > -1)
          rb[iSelected[i]].setSelected(true);
      }
    }
  }

  /** Method setByAction()
   * <p> This method will set the tracks by the Action and data present.
   * @param iAction    = The Type of Profile to display
   * @param stPSI      = Raw Pressure data
   * @param stPSIAvg   = Averaged Pressure data
   * @param stWave     = Raw Seismic data
   * @param stWaveAvg  = Averaged Seismic data
   */

  public void setByAction( int        iAction,
                           seisStruct stPSI,
                           seisStruct stPSIAvg,
                           seisStruct stWave,
                           seisStruct stWaveAvg )
  {
    int i=0;

    this.iAction = iAction;

    if (stPSI != null)
    {
      rb[_PRESSURE_TEST].setEnabled(true);
      rb[_PRESSURE_TEST].setSelected(true);

      rb[_PRESSURE_AVG].setEnabled(true);
      if (stPSIAvg != null)
        rb[_PRESSURE_AVG].setSelected(true);
	}

    if (stWave != null)
    {
      rb[_SEISMIC].setEnabled(true);
      rb[_SEISMIC].setSelected(true);

      rb[_SEISMIC_AVG].setEnabled(true);
      if (stWaveAvg != null)
        rb[_SEISMIC_AVG].setSelected(true);
	}

    for (i=0; i<TOTAL_TRACKS; i++)
    {
      if (rb[i].isEnabled())
        iTotal++;
    }

    iTotal = iTotal+2;
    panel.removeAll();
    if (iTotal < 4)
      panel.setLayout(new GridLayout(4,1));
    else
      panel.setLayout(new GridLayout(iTotal,1));
    panel.setSize(100, iTotal*20);

    for (i=0; i<TOTAL_TRACKS; i++)
    {
      if ((i == seismic.seisTracksStruct._SEISMIC) &&
          (stWave != null))
      {
        pnl[seismic.seisTracksStruct._SRC_SEISMIC].setBackground(
            new Color(seismic.seisTracksStruct.COLORS[_SRC_SEISMIC][0],
                      seismic.seisTracksStruct.COLORS[_SRC_SEISMIC][1],
                      seismic.seisTracksStruct.COLORS[_SRC_SEISMIC][2]));
        lbl[seismic.seisTracksStruct._SRC_SEISMIC].setText(
            seismic.seisTracksStruct.SOURCE[_SRC_SEISMIC][_SRC_COLUMN]);

        panel.add(pnl[seismic.seisTracksStruct._SRC_SEISMIC], null);
        pnl[seismic.seisTracksStruct._SRC_SEISMIC].add(
            lbl[seismic.seisTracksStruct._SRC_SEISMIC], null);
      }

      if ((i == seismic.seisTracksStruct._PRESSURE_TEST) && (stPSI != null))
      {
        pnl[seismic.seisTracksStruct._SRC_PSI].setBackground(
            new Color(seismic.seisTracksStruct.COLORS[_SRC_PSI][0],
                      seismic.seisTracksStruct.COLORS[_SRC_PSI][1],
                      seismic.seisTracksStruct.COLORS[_SRC_PSI][2]));
        lbl[seismic.seisTracksStruct._SRC_PSI].setText(
            seismic.seisTracksStruct.SOURCE[_SRC_PSI][_SRC_COLUMN]);

        panel.add(pnl[seismic.seisTracksStruct._SRC_PSI], null);
        pnl[seismic.seisTracksStruct._SRC_PSI].add(
            lbl[seismic.seisTracksStruct._SRC_PSI], null);
      }

      if (rb[i].isEnabled())
        panel.add(rb[i], null);
    }

    setButtonColors();

    panel.updateUI();
    this.updateUI();
  }

  /** Method setNotify()
   * <p> This method will notify Observers that the panel has changed
   */

  public void setNotify()
  {
    if (notifier != null)
      notifier.notifyObservers(new String("Available Track Panel Changed"));
  }

  /** Method setButtonColors()
   * <p> This method will set change the radio button colors if they are
   *     selected.
   */

  private void setButtonColors()
  {
    int i=0;
    int j=0;
    int k=0;
    int iSource = -1;

    for (i=0; i<seismic.seisTracksStruct.TOTAL_TRACKS; i++)
    {
      for (k=0; k<seismic.seisTracksStruct.TOTAL_SOURCES; k++)
      {
        if (seismic.seisTracksStruct.TRACK[i][_SOURCE].equals(
            seismic.seisTracksStruct.SOURCE[k][_SRC_ID]))
        {
          iSource = k;
        }
      }

      rb[i].setForeground(Color.black);
      rb[i].setBackground(
          new Color(seismic.seisTracksStruct.COLORS[iSource][0],
                    seismic.seisTracksStruct.COLORS[iSource][1],
                    seismic.seisTracksStruct.COLORS[iSource][2]));
      rb[i].setFont(new java.awt.Font("Dialog", 0, 11));
	}
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getTrack()
   * <p> This method will retrieve the last track that was modified
   * @return iTrack = the track number modified
   */

  public int getTrack() { return (iTrack); }

  /** Method getSelected()
   * <p> This method will retrieve the tracks that were selected.
   * @return iSelected = Array of selected tracks
   */

  public int[] getSelected()
  {
    int i      = 0;
    int iCount = 0;

    for (i=0; i<seismic.seisTracksStruct.TOTAL_TRACKS; i++)
    {
      if (rb[i].isSelected())
      {
        iCount++;
      }
    }

    iSelected = new int[iCount];
    iSelect   = 0;
    for (i=0; i<seismic.seisTracksStruct.TOTAL_TRACKS; i++)
    {
      if (rb[i].isSelected())
      {
        iSelected[iSelect] = i;
        iSelect++;
      }
    }

    return (iSelected);
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will set the global variables to null in order to force
   *     JAVA to free the memory allocated for this panel.
   */

  public void close()
  {
    int i=0;

    iSelected = null; // Tracks selected for view

    panel     = null;

    for (i=0; i<ilbl; i++)
    {
      pnl[i] = null;
      lbl[i] = null;
    }

    for (i=0; i<irb; i++)
      rb[i]    = null;
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    for (int i=0; i<seismic.seisTracksStruct.TOTAL_TRACKS; i++)
    {
      if (event.getSource() == rb[i])
      {
        iTrack = i;
        notifier.notifyObservers(new String("Available Track Panel Changed"));
      }
    }
  }
}

/*
 *  @version 1.1 06/01/2016
 *  @author  John Victorine
 */
