/*
 * @psiFrequencyFrame.java Version 1.1 04/01/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package psi.gui;

import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import cmn.cmnStruct;               // Global Common Data Structure
import seismic.seisStruct;          // Seismic Data Structure
//import seismic.gui.seisPlotFrame;   // Seismic Plot Frame

import seismic.seisPlotXYStruct;    // Plot XY Plot Data Structure
import seismic.gui.seisPlotXYFrame; // Plot XY Plot Frame
import plot.plotSymbolStruct;       // Plot Symbol Data Structure
import plot.plotSymbol;             // Plot Symbol Class

import util.utilWaitAnimation;      // Wait Animation Dialog
import util.SwingWorker;            // Swing Worker

/** Class psiFrequencyFrame
 *  <p> This Class is the Frequency Plot Control Frame.
 *
 *  @version 1.1 04/01/2015
 *  @author  John R. Victorine
 */

public class psiFrequencyFrame extends JFrame implements ActionListener
{
  private static final int _PRESSURE_AVG = seismic.seisTracksStruct._PRESSURE_AVG;

  private static final int SCALE[]   = seismic.seisTracksStruct.SCALE;

  private final static int _BOTH     = seismic.plot.seisPlotXYData._BOTH;
  private final static int _RAW      = seismic.plot.seisPlotXYData._RAW;
  private final static int _AVERAGE  = seismic.plot.seisPlotXYData._AVERAGE;

  // Input Variables

  private cmnStruct        stCMN     = null; // Common Data Structure
  private Observable       notifier  = null; // Observables Object

  // Global Variables

  private seisStruct       stData    = null; // Raw Seismic Data
  private seisStruct       stAvg     = null; // Average Seismic Data
//  private seisPlotFrame    plotframe = null; // Seismic Data Plot Frame

  private seisPlotXYFrame  plotXY    = null; // XY Plot Frame
  private seisPlotXYStruct stPlot[]  = null; // Array XY Plot Data Structures
  private int              iPlot     = 0;    // Number of XY Plot Data Structures

  private static final int RAW       = 0;
  private static final int AVERAGE   = 1;

  private double        data_raw[][] = null; // Frequency space Raw Signal
  private double        data_avg[][] = null; // Frequency space Averaged Signal

  private String           sTitle    = "";   // Title for plots

  // -- Time Scale

  private Timestamp tsStart = null; // Starting Date
  private Timestamp tsEnd   = null; // Ending Date

  private Timestamp tsSo    = null; // Original Starting Date
  private Timestamp tsEo    = null; // Original Ending Date

  private static final int START    = 0;
  private static final int END      = 1;
//  private static final int MINIMUM  = 2;
//  private static final int MAXIMUM  = 3;
  private static final int FREQ_MIN = 4;
  private static final int FREQ_MAX = 5;
  private static final int FREQ_PTS = 6;

  private static final int YEAR    = 0;
  private static final int MONTH   = 1;
  private static final int DAY     = 2;
  private static final int HOUR    = 3;
  private static final int MIN     = 4;
  private static final int SEC     = 5;

  int iData = -1;

  // Global Frame Widgets

  // .. Start Date & Time

  private JTextField txtYYYYs    = new JTextField(); // Year Field
  private JTextField txtMMs      = new JTextField(); // Month Field
  private JTextField txtDDs      = new JTextField(); // Day Field

  private JTextField txtHHs      = new JTextField(); // 24 Hour Field
  private JTextField txtmms      = new JTextField(); // minute Field
  private JTextField txtssSs     = new JTextField(); // second Field

  // .. End Date & Time

  private JTextField txtYYYYe    = new JTextField(); // Year Field
  private JTextField txtMMe      = new JTextField(); // Month Field
  private JTextField txtDDe      = new JTextField(); // Day Field

  private JTextField txtHHe      = new JTextField(); // 24 Hour Field
  private JTextField txtmme      = new JTextField(); // minute Field
  private JTextField txtssSe     = new JTextField(); // second Field

  // .. Time Control Buttons

  private JButton    btnReset    = new JButton();    // Reset Button

  // .. Frequency

  private JTextField txtFo       = new JTextField(); // Minimum Frequency
  private JTextField txtFm       = new JTextField(); // Maximum Frequency
  private JTextField txtPts      = new JTextField(); // Total Points to average

  private JButton    btnCompute  = new JButton();    // Compute Filter Button

  // Screen Control Variable

  private int iControlWidth = 350;  // Control Panel Width

  /** Constructor psiFrequencyFrame()
   *  <p> This is the Constructor for this class.
   *  @param stCMN    = Global IQSTRAT Data Structure
   *  @param notifier = Observables
   */

  public psiFrequencyFrame( cmnStruct stCMN, Observable notifier )
  {
    try
    {
      this.stCMN    = stCMN;
      this.notifier = notifier;

      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Panel for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlTop            = new JPanel();
    JPanel pnlHeader         = new JPanel();
    JPanel pnlDateTime       = new JPanel();

    JPanel pnlCenter         = new JPanel();
    JPanel pnlFreq           = new JPanel();

    // Build Other Widgets

    this.setTitle("Frequency Plot Control");
    this.getContentPane().setLayout(new BorderLayout());

    // Panel Top

    pnlTop.setLayout(new BorderLayout());

    // -- Header Panel

    pnlDateTime = BuildDateTimePanel();

    // Panel Center

    pnlCenter.setLayout(new BorderLayout());

    // -- Frequency Panel

    pnlFreq = BuildFrequencyPanel();

    // Add Widgets to Frame

    this.getContentPane().add(pnlTop,    BorderLayout.NORTH);
    pnlTop.add(pnlDateTime,              BorderLayout.CENTER);

    this.getContentPane().add(pnlCenter, BorderLayout.SOUTH);
    pnlCenter.add(pnlFreq,               BorderLayout.NORTH);

   // Display the Frame

    Dimension d  = Toolkit.getDefaultToolkit().getScreenSize();

    this.setSize(new Dimension(iControlWidth, 300));
    this.setLocation((d.width - this.getSize().width) / 2,
                     (d.height - this.getSize().height) / 2);

    this.setResizable(false);
    this.setVisible(true);
  }

  /** Method BuildDateTimePanel()
   *  <p> This method will build the date/time panel on the control dialog
   */

  private JPanel BuildDateTimePanel()
  {
    JPanel pnlDateTime       = new JPanel();

    JPanel pnlDateTimeCenter = new JPanel();

    JPanel pnlDateTimeStart  = new JPanel();
    JPanel pnlDateTimeEnd    = new JPanel();

    JPanel pnlDateStart      = new JPanel();
    JPanel pnlYYYYs          = new JPanel();
    JPanel pnlMMs            = new JPanel();
    JPanel pnlDDs            = new JPanel();

    JPanel pnlTimeStart      = new JPanel();
    JPanel pnlHHs            = new JPanel();
    JPanel pnlmms            = new JPanel();
    JPanel pnlsss            = new JPanel();

    JPanel pnlDateEnd        = new JPanel();
    JPanel pnlYYYYe          = new JPanel();
    JPanel pnlMMe            = new JPanel();
    JPanel pnlDDe            = new JPanel();

    JPanel pnlTimeEnd        = new JPanel();
    JPanel pnlHHe            = new JPanel();
    JPanel pnlmme            = new JPanel();
    JPanel pnlsse            = new JPanel();

    JPanel pnlTimeButtons    = new JPanel();

    TitledBorder titledBorderDates = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Pressure - Frequency Magnitude Plot");
    titledBorderDates.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderStartDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Start Date Time");
    titledBorderStartDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderEndDate = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "End Date Time");
    titledBorderEndDate.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderYYYY = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "YYYY");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMM = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "MM");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderDD = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "DD");
    titledBorderYYYY.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderHH = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "HH");
    titledBorderHH.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBordermm = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "mm");
    titledBordermm.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderssS = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "ss.S");
    titledBorderssS.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    // -------------------------------------------------------
    // ------------------- Date Time Panel -------------------
    // -------------------------------------------------------

    pnlDateTime.setLayout(new BorderLayout());
    pnlDateTime.setBorder(titledBorderDates);

    // -- Date Time Center Panel

    pnlDateTimeCenter.setLayout(new GridLayout(2,1));

    // .......................................................
    // ................... Date Time Start ...................
    // .......................................................

    pnlDateTimeStart.setLayout(new GridLayout());
    pnlDateTimeStart.setBorder(titledBorderStartDate);

    // -- Date Start Panel

    pnlDateStart.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYs.setLayout(new BorderLayout());
    pnlYYYYs.setBorder(titledBorderYYYY);
    pnlYYYYs.add(txtYYYYs,  BorderLayout.CENTER);

    txtYYYYs.setText("");
    txtYYYYs.setColumns(4);
	txtYYYYs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYs.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMs.setLayout(new BorderLayout());
    pnlMMs.setBorder(titledBorderMM);
    pnlMMs.add(txtMMs,  BorderLayout.CENTER);

    txtMMs.setText("");
    txtMMs.setColumns(2);
	txtMMs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMs.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDs.setLayout(new BorderLayout());
    pnlDDs.setBorder(titledBorderDD);
    pnlDDs.add(txtDDs,  BorderLayout.CENTER);

    txtDDs.setText("");
    txtDDs.setColumns(2);
	txtDDs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDs.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateStart.add(pnlYYYYs, BorderLayout.WEST);
    pnlDateStart.add(pnlMMs,   BorderLayout.CENTER);
    pnlDateStart.add(pnlDDs,   BorderLayout.EAST);

    // -- Time Start Panel

    pnlTimeStart.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHs.setLayout(new BorderLayout());
    pnlHHs.setBorder(titledBorderHH);
    pnlHHs.add(txtHHs, BorderLayout.CENTER);

    txtHHs.setText("");
    txtHHs.setColumns(2);
	txtHHs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHs.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmms.setLayout(new BorderLayout());
    pnlmms.setBorder(titledBordermm);
    pnlmms.add(txtmms, BorderLayout.CENTER);

    txtmms.setText("");
    txtmms.setColumns(2);
	txtmms.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmms.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsss.setLayout(new BorderLayout());
    pnlsss.setBorder(titledBorderssS);
    pnlsss.add(txtssSs, BorderLayout.CENTER);

    txtssSs.setText("");
    txtssSs.setColumns(4);
	txtssSs.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSs.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeStart.add(pnlHHs, BorderLayout.WEST);
    pnlTimeStart.add(pnlmms, BorderLayout.CENTER);
    pnlTimeStart.add(pnlsss, BorderLayout.EAST);

    // .......................................................
    // .................... Date Time End ....................
    // .......................................................

    pnlDateTimeEnd.setLayout(new GridLayout());
    pnlDateTimeEnd.setBorder(titledBorderEndDate);

    // -- Date End Panel

    pnlDateEnd.setLayout(new BorderLayout());

    // .. Year Textfield Panel

    pnlYYYYe.setLayout(new BorderLayout());
    pnlYYYYe.setBorder(titledBorderYYYY);
    pnlYYYYe.add(txtYYYYe,  BorderLayout.CENTER);

    txtYYYYe.setText("");
    txtYYYYe.setColumns(4);
	txtYYYYe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtYYYYe.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // .. Month Textfield Panel

    pnlMMe.setLayout(new BorderLayout());
    pnlMMe.setBorder(titledBorderMM);
    pnlMMe.add(txtMMe,  BorderLayout.CENTER);

    txtMMe.setText("");
    txtMMe.setColumns(2);
	txtMMe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtMMe.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // .. Day Textfield Panel

    pnlDDe.setLayout(new BorderLayout());
    pnlDDe.setBorder(titledBorderDD);
    pnlDDe.add(txtDDe,  BorderLayout.CENTER);

    txtDDe.setText("");
    txtDDe.setColumns(2);
	txtDDe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtDDe.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // -- Attach YYYY,MM,DD Panels to Date Start Panel

    pnlDateEnd.add(pnlYYYYe, BorderLayout.WEST);
    pnlDateEnd.add(pnlMMe,   BorderLayout.CENTER);
    pnlDateEnd.add(pnlDDe,   BorderLayout.EAST);

    // -- Time End Panel

    pnlTimeEnd.setLayout(new BorderLayout());

    // .. Hour Textfield Panel

    pnlHHe.setLayout(new BorderLayout());
    pnlHHe.setBorder(titledBorderHH);
    pnlHHe.add(txtHHe, BorderLayout.CENTER);

    txtHHe.setText("");
    txtHHe.setColumns(2);
	txtHHe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtHHe.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // .. Minutes Textfield Panel

    pnlmme.setLayout(new BorderLayout());
    pnlmme.setBorder(titledBordermm);
    pnlmme.add(txtmme, BorderLayout.CENTER);

    txtmme.setText("");
    txtmme.setColumns(2);
	txtmme.setHorizontalAlignment(SwingConstants.TRAILING);
	txtmme.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // .. Seconds Textfield Panel

    pnlsse.setLayout(new BorderLayout());
    pnlsse.setBorder(titledBorderssS);
    pnlsse.add(txtssSe, BorderLayout.CENTER);

    txtssSe.setText("");
    txtssSe.setColumns(4);
	txtssSe.setHorizontalAlignment(SwingConstants.TRAILING);
	txtssSe.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    // -- Attach HH,mm,ss.SS Panels to Date Start Panel

    pnlTimeEnd.add(pnlHHe, BorderLayout.WEST);
    pnlTimeEnd.add(pnlmme, BorderLayout.CENTER);
    pnlTimeEnd.add(pnlsse, BorderLayout.EAST);

    // .......................................................
    // .................. Date Time Buttons ..................
    // .......................................................

    pnlTimeButtons.setLayout(new BorderLayout());

    btnReset.setFont(new java.awt.Font("Dialog", 1, 11));
    btnReset.setText("Reset");
    btnReset.addActionListener(this);

    // Attach Widgets to Panel

    pnlDateTime.add(pnlDateTimeCenter,      BorderLayout.CENTER);
    pnlDateTimeCenter.add(pnlDateTimeStart, null);
    pnlDateTimeStart.add(pnlDateStart,      null);
    pnlDateTimeStart.add(pnlTimeStart,      null);

    pnlDateTimeCenter.add(pnlDateTimeEnd,   null);
    pnlDateTimeEnd.add(pnlDateEnd,          null);
    pnlDateTimeEnd.add(pnlTimeEnd,          null);

    pnlDateTime.add(pnlTimeButtons,         BorderLayout.SOUTH);
    pnlTimeButtons.add(btnReset,            BorderLayout.CENTER);

    return (pnlDateTime);
  }

  /** Method BuildFrequencyPanel()
   *  <p> This method will build the Frequency Control Panel
   */

  private JPanel BuildFrequencyPanel()
  {
	JPanel panel      = new JPanel();
	JPanel pnlCenter  = new JPanel();
	JPanel pnlFo      = new JPanel();
	JPanel pnlFm      = new JPanel();
	JPanel pnlPts     = new JPanel();
	JPanel pnlCompute = new JPanel();

    TitledBorder titledBorderMinimumHz = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Minimum Hz");
    titledBorderMinimumHz.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderMaximumHz = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Maximum Hz");
    titledBorderMaximumHz.setTitleFont(new java.awt.Font("Dialog", 1, 11));

    TitledBorder titledBorderPoints = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Points");
    titledBorderPoints.setTitleFont(new java.awt.Font("Dialog", 1, 11));

	panel.setLayout(new BorderLayout());

    pnlCenter.setLayout(new GridLayout());

    pnlFo.setLayout(new BorderLayout());
    pnlFo.setBorder(titledBorderMinimumHz);
    pnlFo.add(txtFo,                 BorderLayout.CENTER);

    txtFo.setText("0.1");
	txtFo.setHorizontalAlignment(SwingConstants.TRAILING);
	txtFo.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    pnlFm.setLayout(new BorderLayout());
    pnlFm.setBorder(titledBorderMaximumHz);
    pnlFm.add(txtFm,                 BorderLayout.CENTER);

    txtFm.setText("100.0");
	txtFm.setHorizontalAlignment(SwingConstants.TRAILING);
	txtFm.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    pnlPts.setLayout(new BorderLayout());
    pnlPts.setBorder(titledBorderPoints);
    pnlPts.add(txtPts,               BorderLayout.CENTER);

    txtPts.setText("100");
	txtPts.setHorizontalAlignment(SwingConstants.TRAILING);
	txtPts.addFocusListener(new psiFrequencyFrameFocusAdapter(this));

    pnlCompute.setLayout(new BorderLayout());

    btnCompute.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCompute.setText("Compute & Plot Frequency Space");
    btnCompute.addActionListener(this);

    panel.add(pnlCenter,       BorderLayout.CENTER);
    pnlCenter.add(pnlFo,       null);
    pnlCenter.add(pnlFm,       null);
    pnlCenter.add(pnlPts,      null);

    panel.add(pnlCompute,      BorderLayout.SOUTH);
    pnlCompute.add(btnCompute, BorderLayout.CENTER);

	return (panel);
  }

  /* ===================================================================== *
   * ----------------------------- CLOSE --------------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    // Input Variables

    stCMN     = null; // Global IQSTRAT Data Structure
    notifier  = null; // Observables Object

    // Global Variables

    stData    = null; // Seismic Data Structure
    stAvg     = null; // Seismic Data Structure

//    plotframe = null; // Seismic Data Plot Frame

    if (plotXY != null)
      plotXY.close();
    plotXY    = null; // XY Plot Frame

    if (iPlot > 0)
    {
	  for (int i=0; i<iPlot; i++)
	  {
		if (stPlot[i] != null)
		  stPlot[i].delete();
		stPlot[i]  = null;
	  }

	  stPlot  = null;
	}

    // Time Scale

    tsStart   = null; // Starting Date
    tsEnd     = null; // Ending Date

    tsSo      = null; // Original Starting Date
    tsEo      = null; // Original Ending Date

    // Global Frame Widgets

    // .. Start Date & Time

    txtYYYYs    = null; // Year Field
    txtMMs      = null; // Month Field
    txtDDs      = null; // Day Field

    txtHHs      = null; // 24 Hour Field
    txtmms      = null; // minute Field
    txtssSs     = null; // second Field

    // .. End Date & Time

    txtYYYYe    = null; // Year Field
    txtMMe      = null; // Month Field
    txtDDe      = null; // Day Field

    txtHHe      = null; // 24 Hour Field
    txtmme      = null; // minute Field
    txtssSe     = null; // second Field

    // .. Time Control Buttons

    btnReset    = null;    // Reset Button

    // -- Frequency Plot Control Widgets

    txtFo       = null; // Minimum Frequency
    txtFm       = null; // Maximum Frequency
    txtPts      = null; // Total Points to average

    btnCompute  = null; // Compute Fourier Analysis Button

    // -- Track 1 Minimum & Maximum Limits Widgets

//    txtMinimum1 = null; // Minimum Value
//    txtMaximum1 = null; // Maximum Value

    dispose();
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setData(seisStruct st)
   * <p> This method will set the Seismic Data Structure
   * @param st - Seismic Data Structure
   */

  public void setData(seisStruct st)
  {
    this.stData = st;

    if (st != null)
    {
      tsStart = new Timestamp(st.tsStart.getTime()); // Starting Date
      tsEnd   = new Timestamp(st.tsEnd.getTime());   // Ending Date

      tsSo    = new Timestamp(st.tsStart.getTime()); // Starting Date
      tsEo    = new Timestamp(st.tsEnd.getTime());   // Ending Date

      setDateTime(START,  tsStart);
      setDateTime(END,    tsEnd);
    }
  }

  /** Method setAvgData()
   *  <p> This method will set the Averaged Data Set
   *  @return stAvg = the averaged seismic data structure
   */

  public void setAvgData(seisStruct st) { this.stAvg = st; }

  /** Method setData()
   *  <p> This method will set the data to be plotted
   *  @param iData = The data type to be plotted
   */

  public void setData(int iData) { plotXY.setData(iData); }

  /** Method setDateTime()
   *  <p> This method will set the Date and Time Value into either the
   *      Start Date Time or End Date Time.
   *  @param iDate = 0 for Start, 1 for End
   *  @param ts    = timestamp
   */

  private void setDateTime(int iDate, Timestamp ts)
  {
    int i = 0;
    String tokens[]    = null;
    String sDelimiter  = new String("[ ]+");
    String sDelimiter1 = new String("[-]+");
    String sDelimiter2 = new String("[:]+");
    String str         = "";
    String sDate       = "";
    String sTime       = "";
    String date[]      = { "", "", "" };
    String time[]      = { "", "", "" };
    int    iContinue   = 0;
    double d           = 0.0;
    int    id          = 0;

    if (ts != null)
    {
      str = new String(ts.toString());
      tokens = str.split( sDelimiter );
      if (tokens.length > 0)
        sDate = new String(tokens[0].trim());

      tokens = sDate.split( sDelimiter1 );
      if (tokens.length == 3)
      {
  	    for (i=0; i<3; i++)
	    {
	      date[i] = new String(tokens[i]);
	    }

	    iContinue = 1;

        if (iContinue == 1)
	    {
	      iContinue = 0;

	      str = new String(ts.toString());
	      tokens = str.split( sDelimiter );
	      if (tokens.length > 0)
	        sTime = new String(tokens[1].trim());

          tokens = sTime.split( sDelimiter2 );
          if (tokens.length == 3)
          {
  	        for (i=0; i<3; i++)
  	        {
	          time[i] = new String(tokens[i]);
	        }

	        iContinue = 1;

	        if (iContinue == 1)
	        {
	          switch (iDate)
	          {
	            case START:
	             txtYYYYs.setText(date[0]);
	             txtMMs.setText(date[1]);
	             txtDDs.setText(date[2]);

	             txtHHs.setText(time[0]);
	             txtmms.setText(time[1]);
	             txtssSs.setText(time[2]);
	             break;

	           case END:
	             txtYYYYe.setText(date[0]);
	             txtMMe.setText(date[1]);
	             txtDDe.setText(date[2]);

	             txtHHe.setText(time[0]);
	             txtmme.setText(time[1]);
	             txtssSe.setText(time[2]);
	             break;
	          }
	        }
  	      }
	    }
      }
    }
  }

  /** Method setPlot()
   *  <p> This method will pass the plotframe pointer
   *  @plotframe = the plot frame pointer
   *

  public void setPlot(seisPlotFrame plotframe) { this.plotframe = plotframe; }
*/
  /** Method getTitle()
   *  <p> This method will retrieve the title from this dialog
   *  @param sTitle = The title from the frequency analysis
   */

  public void setTitle(String sTitle)
  {
	this.sTitle = new String(sTitle);
  }

  /* ===================================================================== *
   * ------------------------- COMPUTE METHODS --------------------------- *
   * ===================================================================== */

  /** Method Compute()
   * <p> This method will display the Wait Animation for computing the non
   *     linear component
   */

  private void Compute()
  {
	double dFo = 0.0;
	double dFm = 0.0;

    ChangeStatus(false);

	stPlot = new seisPlotXYStruct[2];

    iPlot  = 0;

    if (stData != null)
    {
      ComputeFrequency(RAW);
      iPlot++;
	}

	if (stAvg != null)
	{
      ComputeFrequency(AVERAGE);
      iPlot++;
	}

//    if (stPlot != null)
    if (iPlot > 0)
    {
      if (plotXY != null)
        plotXY.close();
      plotXY    = null; // XY Plot Frame

      if (stPlot[0].iRows > 0)
      {
        plotXY  = new seisPlotXYFrame( iPlot, stPlot );
        plotXY.setTitle1("Pressure from " + stData.location + " - Frequency vs Magnitude Plot");
        plotXY.setTitle2("Red Curve - Raw Data; Green Curve - Averaged Data");
        plotXY.setTitle3( sTitle );
        if (stAvg != null)
        {
          plotXY.setTitle3(stAvg.sData + " - " + stAvg.sData2 + " " + stAvg.sData3);
		}
	  }
    }

    ChangeStatus(true);
  }

  /** Method ChangeStatus()
   *  <p> This method will enable or disable the Compute Frequency Spectrum
   *      panel so the user doesn't accidently change any of the values that
   *      would interrupt the frequency computations
   */

  private void ChangeStatus(boolean b)
  {
    // -- Frequency Spectrum Plot Panel back ground time range

    // .. Start Date & Time  - Whole

    txtYYYYs.setEditable(b);
    txtMMs.setEditable(b);
    txtDDs.setEditable(b);

    txtHHs.setEditable(b);
    txtmms.setEditable(b);
    txtssSs.setEditable(b);

    // .. End Date & Time  - Whole

    txtYYYYe.setEditable(b);
    txtMMe.setEditable(b);
    txtDDe.setEditable(b);

    txtHHe.setEditable(b);
    txtmme.setEditable(b);
    txtssSe.setEditable(b);
  }

  /** Method ComputeFrequency()
   * <p> This method will display the Wait Animation for computing the non
   *     linear component
   *  @param iType = the type of frequency spectrum to make
   */

  private void ComputeFrequency(int iType)
  {
    Timestamp        tso      = null; // Starting Date
    Timestamp        tse      = null; // Ending Date
    SimpleDateFormat dfm      = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
    String           sText    = "";
    long             ltime    = (long) 0;
	double           dFo      = 0.0;
	double           dFm      = 0.0;
	int              iPts     = 0;

    sText   = new String( txtYYYYs.getText() + "-" +
                          txtMMs.getText()   + "-" +
   	                      txtDDs.getText()   + " " +
	    	              txtHHs.getText()   + ":" +
	    	              txtmms.getText()   + ":" +
	    	              txtssSs.getText() );

    try
    {
      ltime   = dfm.parse(sText).getTime();
      tso = new Timestamp( ltime );
    }
    catch (ParseException e)
    {
      e.printStackTrace();
    }

    sText   = new String( txtYYYYe.getText() + "-" +
	  	                  txtMMe.getText()   + "-" +
	   	                  txtDDe.getText()   + " " +
	   	                  txtHHe.getText()   + ":" +
	   	                  txtmme.getText()   + ":" +
	   	                  txtssSe.getText() );

    try
    {
      ltime   = dfm.parse(sText).getTime();
      tse   = new Timestamp( ltime );
	}
    catch (ParseException e)
    {
      e.printStackTrace();
    }

    if (cmn.cmnString.isNumeric(txtFo.getText()))
    {
      dFo = cmn.cmnString.stringToDouble(txtFo.getText());
      if (dFo == 0.0)
      {
        dFo = 0.1;
        txtFo.setText("" + dFo);
	  }
	}

    if (cmn.cmnString.isNumeric(txtFm.getText()))
      dFm = cmn.cmnString.stringToDouble(txtFm.getText());

    if (cmn.cmnString.isNumeric(txtPts.getText()))
      iPts = cmn.cmnString.stringToInt(txtPts.getText());

    switch (iType)
    {
      case RAW:
        data_raw  = psi.math.psiMath.frequencySpace( stData, tso, tse,
                                                     dFo,    dFm, iPts );
        stPlot[0] = loadData(data_raw, dFo, dFm);
        break;
      case AVERAGE:
        data_avg  = psi.math.psiMath.frequencySpace( stAvg, tso, tse,
                                                     dFo,   dFm, iPts );
        stPlot[1] = loadData(data_avg, dFo, dFm);
        break;
	}
  }

  private static seisPlotXYStruct loadData(double d[][], double dFo, double dFm)
  {
	int              i     = 0;
    seisPlotXYStruct st    = new seisPlotXYStruct();
    int              iMin  = 0;
    int              iMax  = 0;
    double           dMin  = 0.0;
    double           dMax  = 0.0;
    double           dTemp = 0.0;

	if (d != null)
	{
	  if (d.length > 0)
	  {
        dMin   = Math.log(dFo)/Math.log(10.0);
        dMax   = Math.ceil(Math.log(dFm)/Math.log(10.0));

	    dMin = Math.floor(dMin);
        iMin = (int) dMin;

        st.iXLog       = 1;            // Plot Log
        st.iXCycles    = (int) dMax - iMin;  // Number of Cycles
        st.dXMaximum   = Math.pow(10.0, (int) dMax); //dFm;          // Maximum X-Axis
        st.dXMinimum   = Math.pow(10.0, iMin); //dFo;          // Minimum X-Axis
        st.dXIncrement = 1.0;          // Increment along the X-Axis
        st.sXMnemonic  = new String("FREQ");           // X-Axis Mnemonic
        st.sXAxis      = new String("Frequency (Hz)"); // X-Axis Label.
//System.out.println(st.iXCycles+" "+st.dXMaximum+" "+st.dXMinimum);

        for (i=0; i<d.length; i++)
        {
	      if (i==0)
	      {
	    	dMin = d[i][1];
	    	dMax = d[i][1];
	      }

	      if (d[i][1] < dMin) { dMin = d[i][1]; }
	      if (d[i][1] > dMax) { dMax = d[i][1]; }
	    }

        if (dMin == dMax)
        {
	      dMax = dMin + 1.0;
	    }

        iMin = -1 + (int) ( Math.log(dMin)/Math.log(10.0) );
        iMax =  1 + (int) ( Math.log(dMax)/Math.log(10.0) );

        st.iYLog       = 1;                       // Plot Log
        st.iYCycles    = iMax - iMin;             // Number of Cycles
        st.dYMaximum   = Math.pow(10.0, iMax);    // Maximum X-Axis
        st.dYMinimum   = Math.pow(10.0, iMin);    // Minimum X-Axis
        st.dYIncrement = 1.0;
//    st.dYIncrement = (int) (dMax-dMin)/10.0;  // Increment along the X-Axis
        st.sYMnemonic  = new String("MAG");       // X-Axis Mnemonic
        st.sYAxis      = new String("Magnitude"); // X-Axis Label.
//System.out.println(st.iYCycles+" "+st.dYMaximum+" "+st.dYMinimum);

        // Plot Data.

        st.iRows     = d.length;             // Number of Rows
        st.depths    = new double[st.iRows]; // Array of Depths
        st.dX        = new double[st.iRows]; // Array of Horizontal Axis Data
        st.dY        = new double[st.iRows]; // Array of Vertical Axis Data
        st.dZ        = new double[st.iRows]; // Array of Other Data
        st.sZ        = new String[st.iRows]; // Data descriptor array for filtering
        st.stSymbols = new plotSymbolStruct[st.iRows]; // Array of Symbol & colors

        for (i=0; i<st.iRows; i++)
        {
          st.depths[i]    = 0.0;
          st.dX[i]        = d[i][0];
          st.dY[i]        = d[i][1];
          st.dZ[i]        = d[i][2];
          st.sZ[i]        = new String("");
          st.stSymbols[i] = plot.plotSymbol.getPlotSymbolStructure(
                                  plot.plotSymbolStruct.FILLED,
                                  plot.plotSymbolStruct.OVAL,
                                  0, 0, 0, "NONE");
	    }
	  }
	}

	return (st);
  }

  /** Method WaitCompute()
   * <p> This method will display the Wait Animation for computing the non
   *     linear component
   */

  private void WaitCompute()
  {
    java.net.URL url = null;

    // Get the URL Path to display the Wait Animation

    try
    {
      url = new java.net.URL(cmn.cmnStruct.IQSTRAT_BASE + "/");
    }
    catch(Exception ex)
    {
      System.out.println("Wrong Image Directory" + ex);
    }

    // Start the Wait Animation Dialog

    final utilWaitAnimation pDialog = new utilWaitAnimation(url);
    pDialog.startAnimation();

    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        Compute();

        return "OK";
      }

      //Runs on the event-dispatching thread.

      public void finished() { pDialog.stopAnimation(); }
    };

    // Start the Wait Animation Thread

    worker.start();  //required for SwingWorker
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnReset)    // Reset Button
    {
      setDateTime(START, tsSo);
      setDateTime(END,   tsEo);
	}

    if (event.getSource() == btnCompute) { WaitCompute(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	focusLostTime(e);
	focusLostFrequency(e);
  }

  /** METHOD focusLostTime()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostTime(FocusEvent e)
  {
    int     iNumeric = 0;
    int     len      = 0;
    int     iField   = -1;
    boolean bVal     = true;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     id       = 0;
    double  d        = 0.0;
    double  d1       = 0.0;

    if (e.getSource() == txtYYYYs)
    {
      iData    = START;
      iField   = YEAR;
      iNumeric = 1;
      sTemp    = new String(txtYYYYs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Year Value is a Numeric Field");
    }

    if (e.getSource() == txtMMs)
    {
      iData    = START;
      iField   = MONTH;
      iNumeric = 1;
      sTemp    = new String(txtMMs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Month Value is a Numeric Field");
    }

    if (e.getSource() == txtDDs)
    {
	  iData    = START;
	  iField   = DAY;
      iNumeric = 1;
      sTemp    = new String(txtDDs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Day Value is a Numeric Field");
    }

    if (e.getSource() == txtHHs)
    {
      iData    = START;
      iField   = HOUR;
      iNumeric = 1;
      sTemp    = new String(txtHHs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Hour Value is a Numeric Field");
    }

    if (e.getSource() == txtmms)
    {
      iData    = START;
      iField   = MIN;
      iNumeric = 1;
      sTemp    = new String(txtmms.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Minutes Value is a Numeric Field");
    }

    if (e.getSource() == txtssSs)
    {
      iData    = START;
      iField   = SEC;
      iNumeric = 1;
      sTemp    = new String(txtssSs.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Start Seconds Value is a Numeric Field");
    }

    if (e.getSource() == txtYYYYe)
    {
      iData    = END;
      iField   = YEAR;
      iNumeric = 1;
      sTemp    = new String(txtYYYYe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("Emd Year Value is a Numeric Field");
    }

    if (e.getSource() == txtMMe)
    {
      iData    = END;
      iField   = MONTH;
      iNumeric = 1;
      sTemp    = new String(txtMMe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Month Value is a Numeric Field");
    }

    if (e.getSource() == txtDDe)
    {
      iData    = END;
      iField   = DAY;
      iNumeric = 1;
      sTemp    = new String(txtDDe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Day Value is a Numeric Field");
    }

    if (e.getSource() == txtHHe)
    {
      iData    = END;
      iField   = HOUR;
      iNumeric = 1;
      sTemp    = new String(txtHHe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Hour Value is a Numeric Field");
    }

    if (e.getSource() == txtmme)
    {
      iData    = END;
      iField   = MIN;
      iNumeric = 1;
      sTemp    = new String(txtmme.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Minutes Value is a Numeric Field");
    }

    if (e.getSource() == txtssSe)
    {
      iData    = END;
      iField   = SEC;
      iNumeric = 1;
      sTemp    = new String( txtssSe.getText().trim());
      len      = sTemp.length();
      sMessage = new String("End Seconds Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        switch (iData)
        {
          case START:
            setDateTime(START, tsStart);
            break;

          case END:
            setDateTime(END,   tsEnd);
            break;
	    }
      }
      else
      {
        if (cmn.cmnString.isNumeric(sTemp))
        {
          switch (iField)
          {
            case YEAR:
            case MONTH:
            case DAY:
            case HOUR:
            case MIN:
    	      id = cmn.cmnString.stringToInt(sTemp);
              break;
            case SEC:
              d  = cmn.cmnString.stringToDouble(sTemp);
              break;
 	  }
        }

        switch (iData)
        {
          case START:
            switch (iField)
            {
              case YEAR:
                if (len != 4) { bVal = false; }
                break;
              case MONTH:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 12)) { bVal = false; }
                else if (len < 2) { txtMMs.setText("0"+sTemp); }
                break;
              case DAY:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 31)) { bVal = false; }
                else if (len < 2) { txtDDs.setText("0"+sTemp); }
                break;
              case HOUR:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 24)) { bVal = false; }
                else if (len < 2) { txtHHs.setText("0"+sTemp); }
                break;
              case MIN:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 60)) { bVal = false; }
                else if (len < 2) { txtmms.setText("0"+sTemp); }
                break;
              case SEC:
                if ((d < 0.0) || (d >= 60.0 )) { bVal = false; }
                break;
	    }

	    if (!bVal)
              setDateTime(START, tsStart);
            break;

          case END:
  	    switch (iField)
            {
              case YEAR:
                if (len != 4) { bVal = false; }
                  bVal = false;
                break;
              case MONTH:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 12)) { bVal = false; }
                else if (len < 2) { txtMMe.setText("0"+sTemp); }
                break;
              case DAY:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 31)) { bVal = false; }
                else if (len < 2) { txtDDe.setText("0"+sTemp); }
                break;
              case HOUR:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 24)) { bVal = false; }
                else if (len < 2) { txtHHe.setText("0"+sTemp); }
                break;
              case MIN:
                if (len > 2) { bVal = false; }
                else if ((id < 0) || (id >= 60)) { bVal = false; }
                else if (len < 2) { txtmme.setText("0"+sTemp); }
                break;
              case SEC:
                if ((d < 0.0) || (d >= 60.0 )) { bVal = false; }
                break;
	    }

	    if (!bVal)
              setDateTime(END,   tsEnd);
            break;
 	    }
      }
    }
  }

  /** METHOD focusLostFrequency()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLostFrequency(FocusEvent e)
  {
    int     iNumeric = 0;
    int     len      = 0;
    int     iField   = -1;
    boolean bVal     = true;
    String  sMessage = new String("");
    String  sTemp    = new String("");
    int     id       = 0;
    double  d        = 0.0;
    double  d1       = 0.0;

    if (e.getSource() == txtFo)
    {
	  iData    = FREQ_MIN;
      iNumeric = 1;
      sTemp    = new String( txtFo.getText().trim());
      sMessage = new String("Minimum Frequency Value is a Numeric Field");
	}

    if (e.getSource() == txtFm)
    {
	  iData    = FREQ_MAX;
      iNumeric = 1;
      sTemp    = new String( txtFm.getText().trim());
      sMessage = new String("Maximum Frequency Value is a Numeric Field");
	}

    if (e.getSource() == txtPts)
    {
	  iData    = FREQ_PTS;
      iNumeric = 1;
      sTemp    = new String( txtPts.getText().trim());
      sMessage = new String("Number of Points Value is a Numeric Field");
	}

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);

        switch (iData)
        {
          case FREQ_MIN:
            txtFo.setText("0.1");
            break;
          case FREQ_MAX:
            txtFm.setText("100.0");
            break;
          case FREQ_PTS:
            txtPts.setText("100");
            break;
		}
      }
	  else
      {
        switch (iData)
        {
          case FREQ_MIN:
            d = cmn.cmnString.stringToDouble( sTemp );
            if (d == 0.0) { txtFo.setText("0.1"); }
            break;
          case FREQ_MAX:
          case FREQ_PTS:
            break;
		}
	  }
	}
  }
}

/** CLASS psiFrequencyFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class psiFrequencyFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  psiFrequencyFrame adaptee;

  psiFrequencyFrameFocusAdapter(psiFrequencyFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/*
 *  @version 1.1 04/01/2015
 *  @author  John Victorine
 */

