/*
 * @quakeTablePanel.java Version 1.1 5/27/2016
 *
 * Copyright (c) 2016 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.Observable;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.sql.Timestamp;

import quake.gui.quakeTable;
import quake.quakeSeisStruct;  // Seismic Data Structure

/** Class quakeTablePanel
 *  <p> This Class will allow the user to select the status of the data
 *
 *  @version 1.1 5/27/2016
 *  @author  John R. Victorine
 */

public class quakeTablePanel extends JPanel implements ActionListener
{
  private Observable      notifier = null; // Observable notifier
  private quakeSeisStruct st       = null; // Seismic Events Catalog
  private Timestamp       tsStart  = null; // Starting Timestamp
  private Timestamp       tsEnd    = null; // Ending Timestamp

  private JFrame     parent = null;

  private quakeTable pTable = null;

  private JButton btnSelect = new JButton();
  private JButton btnCancel = new JButton();

  /** Constructor quakeTablePanel()
   *  <p> This is the Constructor for this class.
   *  @param notifier = Observable notifier
   */

  public quakeTablePanel( Observable notifier, quakeSeisStruct st,
                          Timestamp tsStart,   Timestamp tsEnd )
  {
    try
    {
      this.notifier = notifier;
      this.st       = st;
      this.tsStart  = new Timestamp(tsStart.getTime());
	  this.tsEnd    = new Timestamp(tsEnd.getTime());

      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
    JPanel pnlButtons  = new JPanel();
    JPanel pnl         = new JPanel();
    JScrollPane scroll = new JScrollPane();

    TitledBorder titledBorderCatalog = new TitledBorder(
        new EtchedBorder(EtchedBorder.RAISED,
                         Color.white,
                         new Color(165, 163, 151)),
        "Seismic Events Catalog");
    titledBorderCatalog.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    this.setLayout(new BorderLayout());
    this.setBorder(titledBorderCatalog);

    pnlButtons.setBorder(BorderFactory.createEtchedBorder());

    btnSelect.setFont(new java.awt.Font("Dialog", 1, 11));
    btnSelect.setText("Plot Seismic Record");
    btnSelect.addActionListener(this);

    btnCancel.setFont(new java.awt.Font("Dialog", 1, 11));
    btnCancel.setText("Cancel");
    btnCancel.addActionListener(this);

    pnl.setLayout(new BorderLayout());

    pTable = new quakeTable(st, tsStart, tsEnd);
    scroll = pTable.getScrollPane();

    // Attach Widgets to frame

    this.add(pnlButtons, BorderLayout.SOUTH);
    pnlButtons.add(btnSelect,      null);
//    pnlButtons.add(btnCancel,      null);

    this.add(pnl, BorderLayout.CENTER);
    pnl.add(scroll,                BorderLayout.CENTER);
  }

  /* ===================================================================== *
   * ------------------------- CLOSE METHODS ----------------------------- *
   * ===================================================================== */

  /** Method close()
   * <p> This method will close this dialog and all other dialogs opened by
   *     this dialog
   */

  public void close()
  {
    notifier     = null; // Observable notifier
    st           = null; // Seismic Events Catalog
    tsStart      = null; // Starting Timestamp
    tsEnd        = null; // Ending Timestamp

    pTable       = null;
    btnSelect    = null;
    btnCancel    = null;
  }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method getSelectedRow()
   *  <p> This method will return the selected row.
   *  @return iSelectedRow - Selected Row.
   */

  public int getSelectedRow() { return (pTable.getSelectedRow()); }

  /* ===================================================================== *
   * -------------------------- GET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setParent()
   *  <p> This method will set the Frame calling this panel.
   */

  public void setParent(JFrame parent) { this.parent = parent; }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** Method select()
   * <p> This method will close this dialog and all other dialogs opened by
   *     this dialog
   */

  private void select()
  {
    notifier.notifyObservers(new String("Event Selected"));
  }

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnSelect) { select(); }

    if (event.getSource() == btnCancel)
    {
      if (notifier != null)
        notifier.notifyObservers( new String("Close Events Table Frame") );
    }

    if (parent != null) { parent.toBack(); }
  }
}

/*
 *  @version 1.1 5/27/2016
 *  @author  John Victorine
 */
