/*
 * @quakeSeisStruct.java Version 1.1 08/06/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake;

import java.sql.Timestamp;

/** Class quakeSeisStruct
 *  <p> This Class holds the information that was parsed from the miniSeed
 *      file using the IRIS (Incorporated Research Institutions
 *      for Seismology) source code.
 *
 *  @version 1.1 08/06/2015
 *  @author  John R. Victorine
 */

public class quakeSeisStruct
{
  public String network  = new String(""); // Network Code
  public String station  = new String(""); // Station Identifier
  public String location = new String(""); // Location Identifier
  public String channel  = new String(""); // Channel Identifier

  public Timestamp tsStart      = null; // Starting Timestamp
  public Timestamp tsEnd        = null; // Ending Timestamp

  public double    dMaximum     = 0.0; // Maximum Data Value
  public double    dMinimum     = 0.0; // Minimum Data Value

  // Seismic Records

  public int       iRows        = 0;    // Total Number of rows
  public Timestamp ts[]         = null; // Time Stamp of data record
  public Timestamp tsUTC[]      = null; // Time Stamp of data record in UTC
  public double    mag[]        = null; // Data record

  public double    dLatitude[]  = null; // Latitude
  public double    dLongitude[] = null; // Longitude
  public double    depth[]      = null; // Depth

  public String    src[]        = null; // Source

  /** Method delete()
   *  <p> This method deallocates the data to force Java to release memory
   *      allocated for the data
   */

  public void delete()
  {
    network    = null; // Network Code
    station    = null; // Station Identifier
    location   = null; // Location Identifier
    channel    = null; // Channel Identifier

    tsStart    = null; // Starting Timestamp
    tsEnd      = null; // Ending Timestamp

    // Seismic Records

    iRows      = 0;    // Total Number of rows
    ts         = null; // Time Stamp of data record
    tsUTC      = null; // Time Stamp of data record in UTC
    mag        = null; // Data record

    dLatitude  = null; // Latitude
    dLongitude = null; // Longitude
    depth      = null; // Depth

    src        = null; // Source
  }
}

/*
 *  @version 1.1 08/06/2015
 *  @author  John Victorine
 */
