/*
 * @quakeSeisUtility.java Version 1.1 08/06/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake;

import java.sql.Timestamp;

import quake.quakeSeisStruct;  // Seismic Data Structure

/** Class quakeSeisUtility
 *  <p> This is the quake seismic data structure utility class.
 *
 *  @version 1.1 08/06/2015
 *  @author  John R. Victorine
 */

public class quakeSeisUtility
{
  /** Method quakeSeisStruct copy(quakeSeisStruct stOld)
   *  <p> This method will copy the seismic data structure
   *  @param  stOld = Old quake seismic data structure
   *  @return stNew = New quake seismic data structure
   */

  public static quakeSeisStruct copy(quakeSeisStruct stOld)
  {
	quakeSeisStruct stNew = null;

	if (stOld != null)
	{
	  stNew            = new quakeSeisStruct();

      stNew.network    = new String(stOld.network);  // Network Code
      stNew.station    = new String(stOld.station);  // Station Identifier
      stNew.location   = new String(stOld.location); // Location Identifier
      stNew.channel    = new String(stOld.channel);  // Channel Identifier

      stNew.tsStart    = new Timestamp(stOld.tsStart.getTime());
      stNew.tsEnd      = new Timestamp(stOld.tsEnd.getTime());

      stNew.dMaximum   = stOld.dMaximum;             // Maximum Data Value
      stNew.dMinimum   = stOld.dMinimum;             // Minimum Data Value

      // Seismic Records

      stNew.iRows      = stOld.iRows;                // Total Number of rows
      stNew.ts         = new Timestamp[stNew.iRows]; // Time Stamp of data record
      stNew.tsUTC      = new Timestamp[stNew.iRows]; // Time Stamp of data record in UTC
      stNew.mag        = new double[stNew.iRows];    // Data record
      stNew.dLatitude  = new double[stNew.iRows];    // Latitude
      stNew.dLongitude = new double[stNew.iRows];    // Longitude
      stNew.depth      = new double[stNew.iRows];    // Depth

      stNew.src        = new String[stNew.iRows];    // Source

	  if (stNew.iRows > 0)
	  {
		for (int i=0; i<stNew.iRows; i++)
		{
		  stNew.ts[i]         = new Timestamp(stOld.ts[i].getTime());
		  stNew.tsUTC[i]      = new Timestamp(stOld.tsUTC[i].getTime());
		  stNew.mag[i]        = stOld.mag[i];        // Magnitude
          stNew.dLatitude[i]  = stOld.dLatitude[i];  // Latitude
          stNew.dLongitude[i] = stOld.dLongitude[i]; // Longitude
          stNew.depth[i]      = stOld.depth[i];      // Depth

          stNew.src[i]        = new String(stOld.src[i]); // Source
		}
	  }
	}

	return (stNew);
  }

  /** Method quakeSeisStruct copy(quakeSeisStruct stOld)
   *  <p> This method will copy the seismic data structure
   *  @param  stOld = Old quake seismic data structure
   *  @return stNew = New quake seismic data structure
   */

  public static quakeSeisStruct transfer(quakeSeisStruct stOld)
  {
	quakeSeisStruct stNew = null;

	if (stOld != null)
	{
      stNew = copy( stOld );

	  stOld.delete();
	  stOld = null;
	}

	return (stNew);
  }

  /** Method bubbleSort()
   * <p> This method will sort in ascending depth order (lowest to highest)
   * @param  st = data list structure
   * @return st = sorted data list structure
   */

  public static quakeSeisStruct bubbleSort(quakeSeisStruct st)
  {
    boolean swappedOnPrevRun = true;
//    quakeDataStruct stTemp   = null;

    Timestamp ts         = null; // Time Stamp of data record
    Timestamp tsUTC      = null; // Time Stamp of data record
    double    mag        = 0.0; // Data record

    double    dLatitude  = 0.0; // Latitude
    double    dLongitude = 0.0; // Longitude
    double    depth      = 0.0; // Depth

    String    src        = ""; // Source

    if (st != null)
    {
      if (st.iRows > 1)
      {
        while(swappedOnPrevRun)
        {
          // this variable keeps track of whether to continue sorting or exit

          swappedOnPrevRun = false;

          // loop through every element in the array, except for the last one

          for(int i=0; i<st.iRows-1; i++)
          {
            long ltime1 = st.ts[i].getTime();
            long ltime2 = st.ts[i+1].getTime();

            // if current element is greater than the next swap the two elements

            if(ltime1 > ltime2)
            {
              // we don't want the loop to end just yet, we're not done

              swappedOnPrevRun = true;

              // store element i in a temporary variable

//            stTemp = copy(st.stItem[i]);
		      ts         = new Timestamp(st.ts[i].getTime());
		      tsUTC      = new Timestamp(st.tsUTC[i].getTime());
		      mag        = st.mag[i];        // Magnitude
              dLatitude  = st.dLatitude[i];  // Latitude
              dLongitude = st.dLongitude[i]; // Longitude
              depth      = st.depth[i];      // Depth
              src        = new String(st.src[i]); // Source

              // set element i+1 to where i used to be

//              st.stItem[i] = copy(st.stItem[i+1]);
		      st.ts[i]         = new Timestamp(st.ts[i+1].getTime());
		      st.tsUTC[i]      = new Timestamp(st.tsUTC[i+1].getTime());
		      st.mag[i]        = st.mag[i+1];        // Magnitude
              st.dLatitude[i]  = st.dLatitude[i+1];  // Latitude
              st.dLongitude[i] = st.dLongitude[i+1]; // Longitude
              st.depth[i]      = st.depth[i+1];      // Depth
              st.src[i]        = new String(st.src[i+1]); // Source

              // release the old i from temp into i+1 slot

//              st.stItem[i+1] = copy(stTemp);
		      st.ts[i+1]         = new Timestamp(ts.getTime());
		      st.tsUTC[i+1]      = new Timestamp(tsUTC.getTime());
		      st.mag[i+1]        = mag;        // Magnitude
              st.dLatitude[i+1]  = dLatitude;  // Latitude
              st.dLongitude[i+1] = dLongitude; // Longitude
              st.depth[i+1]      = depth;      // Depth
              st.src[i+1]        = new String(src); // Source
            }
          }
        }
      }
    }

    return (st);
  }

  /** Method print()
   *  <p> This method will print the Quake Seismic Data Structure
   */

  public static void print(quakeSeisStruct st)
  {
    if (st != null)
    {
      for (int i=0; i<st.iRows; i++)
      {
        System.out.println(st.ts[i].toString()+" "+st.mag[i]+" "+
                            st.dLatitude[i]+" "+st.dLongitude[i]+" "+st.depth[i] +" "+
                            st.src[i]);
      }

      System.out.println("Min="+st.dMinimum+" Max="+st.dMaximum+" Start="+
                         st.tsStart.toString()+" End="+st.tsEnd.toString());
    }
  }
}

/*
 *  @version 1.1 08/06/2015
 *  @author  John Victorine
 */
