/*
 * @seisTracksStruct.java Version 1.1 03/04/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic;

/** Class seisTracksStruct
 *  <p> This Class holds the Plot Tracks Data Structure.
 *
 *  @version 1.1 03/04/2015
 *  @author  John R. Victorine
 */

public class seisTracksStruct
{
  public static final int PLOT_TITLES  = 100; // Plot Titles
  public static final int TIME_TRACK   = 100; // Time Track Width
  public static final int LABELSTART   = 25;  // Label Start

  // Source Data Types

  public static final int _SRC_PSI      = 0;
  public static final int _SRC_SEISMIC  = 1;
  public static final int _SRC_SS       = 2;
  public static final int _SRC_WAVELET  = 3;

  public static final int _SRC_ID       = 0;
  public static final int _SRC_COLUMN   = 1;

  public static final int TOTAL_SOURCES = 4;
  public static final String SOURCE[][] =
    {
     // SOURCE         Description
      { "PSI",     "Digital Pressure File Curve Data" },
      { "SEIS",    "Digital Seismic File Curve Data" },
      { "SS",      "Synthetic Seismic" },
      { "WAVE",    "Wavelet" },
    };

  public static final int COLORS[][] =  { { 210, 180, 140 },
	                                      { 204, 255, 153 },
                                          {   0, 255, 100 },
                                          {   0, 255, 100 }};

  // Stratigraphic Track Types & Track Widths

  public static final String ICON = "ICON";

  public static final int TOTAL_TYPES = 3;
  public static final String TYPE[][] =
    {
    //   TYPE         Description                         Width
      { "SEIS",    "Seismic Data Track Format",           "200" },
      { "SS",      "Synthetic Seismic Plot Track Format", "100" },
      { "WAVE",    "Wavelet Plot Track Format",           "100" },
    };

  // Track Group Descriptions

  public static final int TOTAL_TRACKS_DESC = 4;
  public static final String TRACKS_DESC[][] =
  {
    { "SEIS",    "Seismic Curve Format"  },
    { "CL",      "Color Image Track Curve Format"  },
    { "SS",      "Synthetic Seismic"  },
    { "WAVE",    "Wavelet" },
  };

  // Depth Scale Plot Variables

  public static final int    SCALE_1   = 0;   //   1 Minute / Inch
  public static final int    SCALE_2   = 1;   //   2 Minutes / Inch
  public static final int    SCALE_5   = 2;   //   5 Minutes / Inch
  public static final int    SCALE_15  = 3;   //  15 Minutes / Inch
  public static final int    SCALE_30  = 4;   //  30 Minutes / Inch
  public static final int    SCALE_60  = 5;   //  60 Minutes / Inch
  public static final int    SCALE_120 = 6;   // 120 Minutes / Inch

//  public static final int SCALE[] = { 1, 2, 5, 10, 20, 50, 100, 200 };
  public static final int SCALE[] = {  60000,  120000,  300000,
                                      900000, 1800000, 3600000, 7200000 };

  // Available Plot Tracks

  public static final int _NONE           = -1;
  public static final int _SEISMIC        = 0;
  public static final int _SEISMIC_AVG    = 1;
  public static final int _SEISMIC_MAG    = 2;
  public static final int _PRESSURE_TEST  = 3;
  public static final int _PRESSURE_AVG   = 4;
  public static final int TOTAL_TRACKS    = 5;

  public static final int _GROUP       = 0;
  public static final int _SOURCE      = 1;
  public static final int _TYPE        = 2;
  public static final int _DESCRIPTION = 3;
  public static final int _STATUS      = 4;

  public static final String TRACK[][] =
  {
    { "SEIS",  "SEIS",  "SEIS",  "Seismic Wave",           "YES"},
    { "SEIS",  "SEIS",  "SEIS",  "Avg Seismic Wave",       "YES"},
    { "SEIS",  "SEIS",  "SEIS",  "Wave Envelope",          "YES"},
    { "SEIS",  "PSI",   "SEIS",  "Pressure Test",          "YES"},
    { "SEIS",  "PSI",   "SEIS",  "Pressure Test Averaged", "YES"},
  };

  public static final String SEIS_TRACK_DESC[] = {
      "Seismic Wave",
      "Average Seismic Wave",
      "Wave Envelope",
      "Pressure",
      "Averaged Pressure",
  };

  public static final int _SEIS    = seismic.seisStandardTools._SEIS;
  public static final int _AVG     = seismic.seisStandardTools._AVG;
  public static final int _MAG     = seismic.seisStandardTools._MAG;
  public static final int _PSI     = seismic.seisStandardTools._PSI;
  public static final int _PSI_AVG = seismic.seisStandardTools._PSI_AVG;

  public static final int SEIS_TRACK[][] =
  {
    {_SEIS,  -1,        -1,      -1,     -1},
    {-1,     _AVG,      -1,      -1,     -1},
    {-1,     -1,        _MAG,    -1,     -1},
    {_PSI,   -1,        -1,      -1,     -1},
    {-1,     _PSI_AVG,  -1,      -1,     -1},
  };
}

/*
 *  @version 1.1 03/04/2015
 *  @author  John Victorine
 */
