/*
 * @seisUtility.java Version 1.1 03/03/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package seismic;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import edu.sc.seis.seisFile.mseed.DataRecord;

import seismic.seisStruct;  // Seismic Data Structure

/** Class seisUtility
 *  <p> This is the seismic data structure utility class for
 *      IRIS (Incorporated Research Institutions for Seismology) Data.
 *
 *  @version 1.1 03/03/2015
 *  @author  John R. Victorine
 */

public class seisUtility
{
  /** Method copyList(List<DataRecord> oldList)
   *  <p> This method will copy the IRIS Data Record Array List
   *  @param  oldList = Old IRIS Data Record Array List
   *  @return newList = New IRIS Data Record Array List
   */

  public static List<DataRecord> copyList(List<DataRecord> oldList)
  {
    List<DataRecord> newList = new ArrayList<DataRecord>();

    if (oldList != null)
    {
      for (DataRecord dr : oldList)
      {
        newList.add(dr);
      }
    }

    return (newList);
  }

  /** Method seisStruct copyData(seisStruct oldStruct)
   *  <p> This method will copy the seismic data structure
   *  @param  oldStruct = Old seismic data structure
   *  @return newStruct = New seismic data structure
   */

  public static seisStruct copyData(seisStruct stOld)
  {
	seisStruct stNew = null;

	if (stOld != null)
	{
	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(stOld.tsStart.getTime());
      stNew.tsEnd    = new Timestamp(stOld.tsEnd.getTime());

      stNew.dMaximum = stOld.dMaximum; // Maximum Data Value
      stNew.dMinimum = stOld.dMinimum; // Minimum Data Value

      // Seismic Records

      stNew.iRows = stOld.iRows;                   // Total Number of rows
      stNew.ts    = new Timestamp[stNew.iRows];    // Time Stamp of data record
      stNew.data  = new double[stNew.iRows];       // Data record

	  if (stNew.iRows > 0)
	  {
		for (int i=0; i<stNew.iRows; i++)
		{
		  stNew.ts[i]   = new Timestamp(stOld.ts[i].getTime());
		  stNew.data[i] = stOld.data[i];
		}
	  }
	}

	return (stNew);
  }

  /** Method seisStruct copyData(seisStruct oldStruct)
   *  <p> This method will copy the seismic data structure
   *  @param  oldStruct = Old seismic data structure
   *  @param  stStart   = Starting Date & Time of data set
   *  @param  stEnd     = Ending Date & Time of data set
   *  @return newStruct = New seismic data structure
   */

  public static seisStruct copyData(seisStruct stOld, Timestamp tsStart, Timestamp tsEnd, int iHours)
  {
	seisStruct stNew  = null;
	int        iCount = 0;
	long       lstart = (long) 0;
	long       lend   = (long) 0;
	long       ltime  = (long) 0;
	int        iStart = 0;
	long       lhours = (long) 3600000;
    Timestamp  tsTemp = null;

	if (stOld != null)
	{
	  stNew          = new seisStruct();

      stNew.network  = new String(stOld.network);  // Network Code
      stNew.station  = new String(stOld.station);  // Station Identifier
      stNew.location = new String(stOld.location); // Location Identifier
      stNew.channel  = new String(stOld.channel);  // Channel Identifier

      stNew.sData    = new String(stOld.sData);    // Type of Filter
      stNew.sData2   = new String(stOld.sData2);   // Specifics of Filtering
      stNew.sData3   = new String(stOld.sData3);   // Specifics of Filtering

      stNew.tsStart  = new Timestamp(tsStart.getTime());
      stNew.tsEnd    = new Timestamp(tsEnd.getTime());

      stNew.ts       = new Timestamp[stOld.iRows];    // Time Stamp of data record
      stNew.data     = new double[stOld.iRows];       // Data record

	  if (stOld.iRows > 0)
	  {
        lstart = tsStart.getTime();
        lend   = tsEnd.getTime();

		for (int i=0; i<stOld.iRows; i++)
		{
		  ltime = stOld.ts[i].getTime() + iHours * lhours;

		  if ((ltime >= lstart) && (ltime <= lend))
		  {
	        tsTemp = new Timestamp(ltime);
if (iCount == 0)
System.out.println(	stOld.ts[i].toString()+" "+tsTemp.toString());

//		    stNew.ts[iCount]   = new Timestamp(stOld.ts[i].getTime());
		    stNew.ts[iCount]   = new Timestamp(ltime);
		    stNew.data[iCount] = stOld.data[i];

  		    if (iStart == 0)
  		    {
		  	  stNew.dMinimum = stNew.data[iCount];
			  stNew.dMaximum = stNew.data[iCount];
			  iStart = 1;
		    }

		    if (stNew.dMinimum > stNew.data[iCount])
		      stNew.dMinimum = stNew.data[iCount];

		    if (stNew.dMaximum < stNew.data[iCount])
		      stNew.dMaximum = stNew.data[iCount];

		    iCount++;
		  }
		}
	  }
      stNew.iRows = iCount;
   	}

	return (stNew);
  }
}

/*
 *  @version 1.1 03/03/2015
 *  @author  John Victorine
 */
